/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;

import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.Umfeldgroessen.*;

/**
 * Fasst Fabrikmethoden zusammen zum Erzeugen von Fuzzy-Sets für Umfelddaten.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class UmfelddatenStandardparameterFactory {

    private UmfelddatenStandardparameterFactory() {
        // utility class
    }

    public static LinguistischeVariable glaette() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("nein", 1, 1, 9, 10));
        fuzzySets.add(FuzzySet.trapez("ja", 9, 10, 12, 12));
        return new LinguistischeVariable(GLAETTE, new Wertebereich(1, 12), fuzzySets);
    }

    public static LinguistischeVariable helligkeit() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("dunkel", 0, 0, 85, 110));
        fuzzySets.add(FuzzySet.trapez("dämmrig", 85, 100, 1800, 2100));
        fuzzySets.add(FuzzySet.trapez("hell", 1800, 2100, 14500, 15500));
        fuzzySets.add(FuzzySet.trapez("sehr hell", 14500, 15500, 60000, 60000));
        return new LinguistischeVariable(HELLIGKEIT, new Wertebereich(0, 60000), fuzzySets);
    }

    public static LinguistischeVariable niederschlagsintensitaet() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("kein", 0, 0, 0, 1));
        fuzzySets.add(FuzzySet.trapez("schwach", 0, 1, 4, 6));
        fuzzySets.add(FuzzySet.trapez("mittel", 4, 6, 8, 12));
        fuzzySets.add(FuzzySet.trapez("stark", 8, 12, 14, 16));
        fuzzySets.add(FuzzySet.trapez("sehr stark", 14, 16, 20, 20));
        return new LinguistischeVariable(NIEDERSCHLAGSINTENSITAET, new Wertebereich(0, 20), fuzzySets);
    }

    public static LinguistischeVariable sichtweite() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("bis 50m", 10, 10, 45, 55));
        fuzzySets.add(FuzzySet.trapez("bis 80m", 45, 55, 75, 85));
        fuzzySets.add(FuzzySet.trapez("bis 120m", 75, 85, 110, 130));
        fuzzySets.add(FuzzySet.trapez("bis 250m", 110, 130, 230, 270));
        fuzzySets.add(FuzzySet.trapez("bis 400m", 230, 270, 370, 430));
        fuzzySets.add(FuzzySet.trapez("über 400m", 370, 430, 2000, 2000));
        return new LinguistischeVariable(SICHTWEITE, new Wertebereich(10, 2000), fuzzySets);
    }

    public static LinguistischeVariable windgeschwindigkeit() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("kein", 0, 0, 0, 1));
        fuzzySets.add(FuzzySet.trapez("schwach", 0, 1, 3, 4));
        fuzzySets.add(FuzzySet.trapez("mittel", 3, 4, 8, 10));
        fuzzySets.add(FuzzySet.trapez("stark", 8, 10, 100, 100));
        return new LinguistischeVariable(WINDGESCHWINDIGKEIT, new Wertebereich(0, 100), fuzzySets);
    }

    public static LinguistischeVariable windrichtung() {
        List<FuzzySet> fuzzySets = new ArrayList<>();
        fuzzySets.add(FuzzySet.trapez("Nord", 0, 0, 20, 25));
        fuzzySets.add(FuzzySet.trapez("Nordost", 20, 25, 65, 70));
        fuzzySets.add(FuzzySet.trapez("Ost", 65, 70, 110, 115));
        fuzzySets.add(FuzzySet.trapez("Südost", 110, 115, 155, 160));
        fuzzySets.add(FuzzySet.trapez("Süd", 155, 160, 200, 205));
        fuzzySets.add(FuzzySet.trapez("Südwest", 200, 205, 245, 250));
        fuzzySets.add(FuzzySet.trapez("West", 245, 250, 290, 295));
        fuzzySets.add(FuzzySet.trapez("Nordwest", 290, 295, 335, 340));
        fuzzySets.add(FuzzySet.trapez("Nord", 335, 340, 359, 359));
        return new LinguistischeVariable(WINDRICHTUNG, new Wertebereich(0, 359), fuzzySets);
    }

    public static Regelbasis regelbasis() {
        List<Regel> regeln = new ArrayList<>();
        fuegeRegelnFuerSichtverhaeltnisseHinzu(regeln);
        fuegeRegelnFuerFahrbahnzustandHinzu(regeln);
        return new Regelbasis(regeln);
    }

    @SuppressWarnings("MethodLength")
    private static void fuegeRegelnFuerSichtverhaeltnisseHinzu(List<Regel> regeln) {
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "sehr hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m"),
                                new TermVariable(SICHTWEITE, "über 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "gut")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "sehr hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m"),
                                new TermVariable(SICHTWEITE, "über 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "gut")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "sehr hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m"),
                                new TermVariable(SICHTWEITE, "über 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "gut")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m"),
                                new TermVariable(SICHTWEITE, "über 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "leicht eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "hell"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dämmrig"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m"),
                                new TermVariable(SICHTWEITE, "über 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "leicht eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dämmrig"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m"),
                                new TermVariable(SICHTWEITE, "bis 400m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "leicht eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dämmrig"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 80m"),
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dämmrig"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 50m"),
                                new TermVariable(SICHTWEITE, "bis 80m"),
                                new TermVariable(SICHTWEITE, "bis 120m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "stark"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dämmrig"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 50m"),
                                new TermVariable(SICHTWEITE, "bis 80m"),
                                new TermVariable(SICHTWEITE, "bis 120m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "sehr stark"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dunkel"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 80m"),
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dunkel"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 80m"),
                                new TermVariable(SICHTWEITE, "bis 120m"),
                                new TermVariable(SICHTWEITE, "bis 250m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "eingeschränkt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dunkel"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 50m"),
                                new TermVariable(SICHTWEITE, "bis 80m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "unzureichend")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dunkel"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 50m"),
                                new TermVariable(SICHTWEITE, "bis 80m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "stark"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "unzureichend")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(HELLIGKEIT, "dunkel"),
                        new Maximum(Arrays.asList(
                                new TermVariable(SICHTWEITE, "bis 50m"),
                                new TermVariable(SICHTWEITE, "bis 80m")
                        )),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "sehr stark"))
                )).dann(new TermVariable(SICHTVERHAELTNISSE, "unzureichend")
                ).erzeuge()
        );
    }

    private static void fuegeRegelnFuerFahrbahnzustandHinzu(List<Regel> regeln) {
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "nein"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "trocken")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "nein"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "nass")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "nein"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "nass")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "nein"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "stark"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "nass")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "nein"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "sehr stark"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "nass")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "ja"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "kein"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "glatt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "ja"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "schwach"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "glatt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "ja"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "mittel"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "glatt")
                ).erzeuge()
        );
        regeln.add(Regel.wenn(
                new Minimum(Arrays.asList(
                        new TermVariable(GLAETTE, "ja"),
                        new TermVariable(NIEDERSCHLAGSINTENSITAET, "stark"))
                )).dann(new TermVariable(FAHRBAHNZUSTAND, "glatt")
                ).erzeuge()
        );
    }

}
