/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain.verkehrsdaten;

import java.util.*;

/**
 * Repräsentiert den Güteindex zwischen 0.0 und 1.0 eines Wertes.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public final class Gueteindex {

    public static final Gueteindex NICHT_ERMITTELBAR = new Gueteindex(-1);
    public static final Gueteindex FEHLERHAFT = new Gueteindex(-2);
    public static final Gueteindex NICHT_ERMITTELBAR_FEHLERHAFT = new Gueteindex(-3);

    private final int wert;

    private Gueteindex(double wert) {
        this.wert = (int) Math.round(wert * 10_000.0);
    }

    public static Gueteindex von(double wert) {
        if (wert < 0 || wert > 1) {
            throw new IllegalArgumentException("Der Güteindex muss zwischen 0 und 1 liegen: " + wert);
        }
        return new Gueteindex(wert);
    }

    public Double getWert() {
        return wert / 10_000.0;
    }

    public boolean isNull() {
        return wert < 0;
    }

    public boolean nonNull() {
        return !isNull();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Gueteindex that = (Gueteindex) o;
        return Objects.equals(wert, that.wert);
    }

    @Override
    public int hashCode() {
        return Objects.hash(wert);
    }

    @Override
    public String toString() {
        if (nonNull()) {
            return String.valueOf(getWert());
        } else {
            if (this == NICHT_ERMITTELBAR) {
                return "nicht ermittelbar";
            } else if (this == FEHLERHAFT) {
                return "fehlerhaft";
            } else if (this == NICHT_ERMITTELBAR_FEHLERHAFT) {
                return "nicht ermittelbar/fehlerhaft";
            } else {
                throw new IllegalStateException("[unreachable code] Nicht behandelter Zustand: " + wert);
            }
        }
    }

}
