/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;

import java.util.*;

/**
 * Empfänger von Messwerten vom Datenverteiler.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
@SuppressWarnings("ClassDataAbstractionCoupling")
public class MesswertEmpfaenger {

    private final Event<Datensatz<?, Collection<Messwert>>> messwertEmpfangen = new Event<>();

    private final VerkehrsdatenKurzzeitMesswertEmpfaenger verkehrsdatenKurzzeitMesswertEmpfaenger;
    private final GlaetteMesswertEmpfaenger glaetteMesswertEmpfaenger;
    private final HelligkeitMesswertEmpfaenger helligkeitMesswertEmpfaenger;
    private final NiederschlagsintensitaetMesswertEmpfaenger niederschlagsintensitaetMesswertEmpfaenger;
    private final SichtweiteMesswertEmpfaenger sichtweiteMesswertEmpfaenger;
    private final WindgeschwindigkeitMesswertEmpfaenger windgeschwindigkeitMesswertEmpfaenger;
    private final WindrichtungMesswertEmpfaenger windrichtungMesswertEmpfaenger;

    public MesswertEmpfaenger(ClientDavInterface verbindung, ObjektFactory objektFactory) {
        verkehrsdatenKurzzeitMesswertEmpfaenger = new VerkehrsdatenKurzzeitMesswertEmpfaenger(verbindung, objektFactory);
        verkehrsdatenKurzzeitMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        glaetteMesswertEmpfaenger = new GlaetteMesswertEmpfaenger(verbindung, objektFactory);
        glaetteMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        helligkeitMesswertEmpfaenger = new HelligkeitMesswertEmpfaenger(verbindung, objektFactory);
        helligkeitMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        niederschlagsintensitaetMesswertEmpfaenger = new NiederschlagsintensitaetMesswertEmpfaenger(verbindung, objektFactory);
        niederschlagsintensitaetMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        sichtweiteMesswertEmpfaenger = new SichtweiteMesswertEmpfaenger(verbindung, objektFactory);
        sichtweiteMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        windgeschwindigkeitMesswertEmpfaenger = new WindgeschwindigkeitMesswertEmpfaenger(verbindung, objektFactory);
        windgeschwindigkeitMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);

        windrichtungMesswertEmpfaenger = new WindrichtungMesswertEmpfaenger(verbindung, objektFactory);
        windrichtungMesswertEmpfaenger.messwertEmpfangen().addHandler(messwertEmpfangen::send);
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        verkehrsdatenKurzzeitMesswertEmpfaenger.anmelden(objekte);
        glaetteMesswertEmpfaenger.anmelden(objekte);
        helligkeitMesswertEmpfaenger.anmelden(objekte);
        niederschlagsintensitaetMesswertEmpfaenger.anmelden(objekte);
        sichtweiteMesswertEmpfaenger.anmelden(objekte);
        windgeschwindigkeitMesswertEmpfaenger.anmelden(objekte);
        windrichtungMesswertEmpfaenger.anmelden(objekte);
    }

    public Event<Datensatz<?, Collection<Messwert>>> messwertEmpfangen() {
        return messwertEmpfangen;
    }

}
