/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;

import java.util.*;

/**
 * Empfänger von Parametern vom Datenverteiler.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class ParameterEmpfaenger {

    private final Event<Datensatz<?, LinguistischeVariable>> fuzzySetsEmpfangen = new Event<>();
    private final Event<Datensatz<?, Regelbasis>> regelbasisEmpfangen = new Event<>();

    private final FuzzySetsEmpfaenger fuzzySetsEmpfaenger;
    private final RegelbasisEmpfaenger regelbasisEmpfaenger;

    public ParameterEmpfaenger(ClientDavInterface verbindung, ObjektFactory objektFactory) {
        fuzzySetsEmpfaenger = new FuzzySetsEmpfaenger(verbindung, objektFactory);
        fuzzySetsEmpfaenger.fuzzySetsEmpfangen().addHandler(fuzzySetsEmpfangen::send);

        regelbasisEmpfaenger = new RegelbasisEmpfaenger(verbindung, objektFactory);
        regelbasisEmpfaenger.regelbasisEmpfangen().addHandler(regelbasisEmpfangen::send);
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        fuzzySetsEmpfaenger.anmelden(objekte);
        regelbasisEmpfaenger.anmelden(objekte);
    }

    public Event<Datensatz<?, LinguistischeVariable>> fuzzySetsEmpfangen() {
        return fuzzySetsEmpfangen;
    }

    public Event<Datensatz<?, Regelbasis>> regelbasisEmpfangen() {
        return regelbasisEmpfangen;
    }

}
