/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.portals;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;
import de.bsvrz.sys.funclib.debug.*;

import java.text.*;
import java.util.*;

import static de.bsvrz.iav.subsegana.subsegana.domain.umfelddaten.Umfeldgroessen.*;

class WindgeschwindigkeitMesswertEmpfaenger extends Kommunikation {

    private final Event<Datensatz<Windgeschwindigkeitssensor, Collection<Messwert>>> messwertEmpfangen = new Event<>();

    private final Debug log = Debug.getLogger();
    private final ClientReceiverInterface receiver = new Receiver();
    private final ObjektFactory objektFactory;

    WindgeschwindigkeitMesswertEmpfaenger(ClientDavInterface verbindung, ObjektFactory objektFactory) {
        super(verbindung);
        this.objektFactory = objektFactory;
    }

    public void anmelden(Collection<Systemobjekt> objekte) {
        Collection<SystemObject> sensoren = konvertiereObjekte(objekte, obj -> obj instanceof Windgeschwindigkeitssensor);
        DataDescription dd = datenbeschreibung("atg.ufdsWindGeschwindigkeitMittelWert", "asp.messWertErsetzung");
        getVerbindung().subscribeReceiver(receiver, sensoren, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        log.config(MessageFormat.format("Als Empfänger von Windgeschwindigkeit für {0} Sensoren angemeldet.", sensoren.size()));
    }

    private Datensatz<Windgeschwindigkeitssensor, Collection<Messwert>> konvertiereDatensatz(ResultData result) {
        Windgeschwindigkeitssensor objekt = konvertiereObjekt(result.getObject());
        Messwert wert = konvertiereMesswert(result.getData());
        Datensatz<Windgeschwindigkeitssensor, Collection<Messwert>> datensatz = new Datensatz<>(konvertiereZeitstempel(result.getDataTime()), objekt, Collections.singletonList(wert));
        log.fine("Messwert für Windgeschwindigkeit empfangen", datensatz);
        return datensatz;
    }

    private <T> T konvertiereObjekt(SystemObject objekt) {
        return (T) objektFactory.gibSystemobjekt(objekt);
    }

    private Messwert konvertiereMesswert(Data data) {
        Data.NumberValue wert = data.getItem(WINDGESCHWINDIGKEIT).getScaledValue("Wert");
        if (wert.isNumber()) {
            return new Messwert(WINDGESCHWINDIGKEIT, wert.doubleValue());
        } else {
            return new Messwert(WINDGESCHWINDIGKEIT);
        }
    }

    Event<Datensatz<Windgeschwindigkeitssensor, Collection<Messwert>>> messwertEmpfangen() {
        return messwertEmpfangen;
    }

    private class Receiver implements ClientReceiverInterface {

        @Override
        public void update(ResultData[] results) {
            for (ResultData r : results) {
                if (r.hasData()) {
                    messwertEmpfangen.send(konvertiereDatensatz(r));
                }
            }
        }

    }

}
