/*
 * Segment 5 Intelligente Analyseverfahren, SW-Einheit 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007-2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.domain;

import de.bsvrz.iav.fuzzylib.fuzzylib.*;
import de.bsvrz.iav.subsegana.subsegana.domain.shared.*;
import de.bsvrz.iav.subsegana.subsegana.shared.*;

import java.util.*;
import java.util.stream.*;

/**
 * Fuzzyfiziert Messwerte.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Fuzzyfizierung {

    private final Event<Collection<Datensatz<?, FuzzyVariable>>> messwerteFuzzyfiziert = new Event<>();

    private Map<DatensatzSchluessel, LinguistischeVariable> fuzzySets = new LinkedHashMap<>();

    public void fuzzySetsAktualisiert(Datensatz<?, LinguistischeVariable> parameter) {
        DatensatzSchluessel schluessel = new DatensatzSchluessel(
                parameter.getSystemobjekt().getPid(),
                parameter.getDatum().getName()
        );
        if (parameter.getDatum().nonNull()) {
            fuzzySets.put(schluessel, parameter.getDatum());
        } else {
            fuzzySets.remove(schluessel);
        }
    }

    public void fuzzyfiziere(Datensatz<?, Collection<Messwert>> datensatz) {
        Collection<Datensatz<?, FuzzyVariable>> fuzzyWerte = datensatz.getDatum().stream()
                .map(m -> fuzzyfiziere(datensatz.getSystemobjekt(), m))
                .filter(Objects::nonNull)
                .map(m -> new Datensatz<>(datensatz.getZeitstempel(), datensatz.getSystemobjekt(), m))
                .collect(Collectors.toList());
        if (fuzzyWerte.isEmpty()) {
            return;
        }
        messwerteFuzzyfiziert.send(fuzzyWerte);
    }

    private FuzzyVariable fuzzyfiziere(Systemobjekt objekt, Messwert messwert) {
        DatensatzSchluessel schluessel = new DatensatzSchluessel(
                objekt.getPid(),
                messwert.getName()
        );
        LinguistischeVariable linguistischeVariable = fuzzySets.get(schluessel);
        if (linguistischeVariable == null) {
            return null;
        } else if (messwert.nonNull()) {
            return linguistischeVariable.fuzzyfiziere(messwert.getWert());
        } else {
            return linguistischeVariable.erzeugeFuzzyVariable();
        }
    }

    public Event<Collection<Datensatz<?, FuzzyVariable>>> messwerteFuzzyfiziert() {
        return messwerteFuzzyfiziert;
    }

    private static class DatensatzSchluessel {

        private final String systemobjekt;
        private final String variable;

        DatensatzSchluessel(String systemobjekt, String variable) {
            this.systemobjekt = systemobjekt;
            this.variable = variable;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            DatensatzSchluessel that = (DatensatzSchluessel) o;
            return Objects.equals(systemobjekt, that.systemobjekt) &&
                    Objects.equals(variable, that.variable);
        }

        @Override
        public int hashCode() {
            return Objects.hash(systemobjekt, variable);
        }

        @Override
        public String toString() {
            return "DatensatzSchluessel{" +
                    "systemobjekt='" + systemobjekt + '\'' +
                    ", variable='" + variable + '\'' +
                    '}';
        }

    }

}
